@@ -28,7 +28,7 @@ class BaseMonitor(object):
   NAME = 'base'
   TYPE = None
   UID = None  # Can be set to any value guaranteed to be unique, a uuid.uuid4() is recommended
-  SOURCES = {}
+  SOURCES = None
 
   def __init__(self, backoff=None, logger=None):
     """Initialize the monitor.
@@ -58,10 +58,6 @@ class BaseMonitor(object):
     self.alert_delays = {}
     self.livestreams = {}
 
-    # Store just the source names separately to prevent having to repeatedly
-    # look them up
-    self.monitored_source_names = set(self.SOURCES.keys())
-
   def store_data_point(self, data):
     """Puts a single data point into the database.
 
@@ -74,11 +70,6 @@ class BaseMonitor(object):
 
     monitor_data = copy.copy(data)
     monitor_data.pop('extra', None)
-
-    if set(monitor_data.keys()) != self.monitored_source_names:
-      raise AttributeError('The returned data does not match the '
-                           'source list for monitor %s' % self.NAME)
-
     monitor_data['timestamp'] = time.time()
     self.data_points.append(monitor_data)
 
@@ -225,6 +216,8 @@ class UsageMonitor(IntervalMixin, BaseMonitor):
   """Base class for all "usage" type monitors."""
 
   TYPE = 'usage'
+  SOURCES = {}
+  DEFAULT_TYPE = None
 
   def alert_check(self, data):
     """Checks the monitored data to determine if an alert should be created.
@@ -236,7 +229,7 @@ class UsageMonitor(IntervalMixin, BaseMonitor):
 
     now = time.time()
 
-    for source in self.monitored_source_names:
+    for source in self.SOURCES.keys():
       delay = self.alert_delays.get(source)
       threshold = self.expected_values.get(source)
       usage = data.get(source)
@@ -252,7 +245,9 @@ class UsageMonitor(IntervalMixin, BaseMonitor):
         self.alert_counters[source] += 1
         if self.alert_counters[source] >= self.FLOP_PROTECTION_COUNTER:
           # Send an alert to the server with any extra information for this source
-          extra = data.pop('extra', {})
+          extra_data = data.pop('extra', {}).get(source, {})
+          extra_alert_data = self.extra_alert_data(source)
+          extra_data.update(extra_alert_data)
           alert = {
               'threshold': threshold,
               'current_usage': usage,
@@ -264,7 +259,7 @@ class UsageMonitor(IntervalMixin, BaseMonitor):
                   'source': source,
                   'source_type': self.SOURCES[source],
               },
-              'event_data': self.extra_alert_data(source, extra) or {},
+              'event_data': extra_data
           }
           self.send_alert(alert)
 
@@ -281,7 +276,48 @@ class LogMonitor(BaseMonitor):
   TYPE = 'log'
 
 
-class StatusMonitor(BaseMonitor):
+class StatusMonitor(IntervalMixin, BaseMonitor):
   """Base class for all "status" type monitors."""
 
   TYPE = 'status'
+  STATES = []
+  SOURCES = []
+
+  def alert_check(self, data):
+    """Checks to see if the monitored sources are in the expected state.
+
+    Args:
+      data: dictonary of source to its state.
+    """
+
+    now = time.time()
+
+    for source in self.SOURCES:
+      delay = self.alert_delays.get(source)
+      expected_state = self.expected_values.get(source)
+      current_state = data.get(source)
+
+      if delay and delay <= now:
+        self.alert_delays.pop(delay)
+        delay = None
+
+      if expected_state and not delay and expected_state != current_state:
+        self.logger.debug('Current %s state is %s, expected state is %s', source,
+                          current_state, expected_state)
+
+        extra = data.pop('extra', {})
+        alert = {
+            'expected_state': expected_state,
+            'current_state': current_state,
+            'timestamp': now,
+            'monitor': {
+                'id': self.UID,
+                'name': self.NAME,
+                'type': self.TYPE,
+                'source': source,
+            },
+            'event_data': self.extra_alert_data(source, extra) or {},
+        }
+        self.send_alert(alert)
+        # Put a delay on the next alert to prevent a flood of alert messages
+        self.alert_delays[source] = now + self.backoff
